@@ -11,18 +11,14 @@
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms same terms as perl itself.
 #
-#  $Header$
 use strict;
 
 package Data::FormValidator::Results;
 
 use Data::FormValidator::Filters qw/:filters/;
 use Data::FormValidator::Constraints (qw/:validators :matchers/);
-use vars qw/$AUTOLOAD $VERSION/;
 use Symbol;
 
-$VERSION = 3.49_1;
-
 =pod
 
 =head1 NAME
@@ -31,7 +27,7 @@ Data::FormValidator::Results - Object which contains the results of an input val
 
 =head1 SYNOPSIS
 
- 	my $results = Data::FormValidator->check(\%input_hash, \%dfv_profile);
+    my $results = $validator->check( \%fdat, "customer_infos" );
 
     # Print the name of missing fields
     if ( $results->has_missing ) {
@@ -92,11 +88,9 @@ sub _process {
 	# msgs() method will need access to the profile
 	$self->{profile} = $profile;
 
-	my %imported_validators;
-
     # import valid_* subs from requested packages
 	foreach my $package (_arrayify($profile->{validator_packages})) {
-		if ( !exists $imported_validators{$package} ) {
+		if ( !exists $profile->{imported_validators}{$package} ) {
 			eval "require $package";
 			if ($@) {
 				die "Couldn't load validator package '$package': $@";
@@ -113,7 +107,7 @@ sub _process {
 					*{qualify_to_ref($sub)} = $subref;
 				}
 			}
-			$imported_validators{$package} = 1;
+			$profile->{imported_validators}{$package} = 1;
 		}
 	}
 
@@ -124,7 +118,7 @@ sub _process {
 			$filter = (ref $filter ? $filter : *{qualify_to_ref("filter_$filter")}{CODE}) ||
 				die "No filter found named: '$filter'";
 			foreach my $field ( keys %valid ) {
-				# apply filter, modifying %valid by reference, skipping undefined values
+				# apply filter, modifying %valid by reference
 				_filter_apply(\%valid,$field,$filter);
 			}
 		}	
@@ -196,12 +190,12 @@ sub _process {
 		if (ref $valid{$field}) {
 			if ( ref $valid{$field} eq 'ARRAY' ) {
 				for (my $i = 0; $i < scalar @{ $valid{$field} }; $i++) {
-					$valid{$field}->[$i] = undef unless (defined $valid{$field}->[$i] and length $valid{$field}->[$i]);
+					$valid{$field}->[$i] = undef unless length $valid{$field}->[$i];
 				}
 			}
 		}
 		else {
-			delete $valid{$field} unless (defined $valid{$field} and length $valid{$field});
+			delete $valid{$field} unless length $valid{$field};
 		}
 	}
 
@@ -604,85 +598,110 @@ sub msgs {
 
 =pod
 
-=head1 meta()
+=head1 WRITING YOUR OWN VALIDATION ROUTINES
 
-In a few cases, a constraint may discover meta data that is useful
-to access later. For example, when using L<Data::FormValidator::Constraints::Upload>, several bits of meta data are discovered about files in the process
-of validating. These can include "bytes", "width", "height" and "extension".
-The C<meta()> function is used by constraint methods to set this data. It's
-also used to access this data. Here are some examples.
+It's easy to create your own module of validation routines. The easiest approach
+to this may be to check the source code of the Data::FormValidator module for example
+syntax. Also notice the C<validator_packages> option in the input profile.
 
- # return all field names that have meta data
- my @fields = $results->meta();
+You will find that validation routines are named two ways. Some are named with
+the prefix C<match_> while others start with C<valid_>. The difference is that the
+C<match_ routines> are built to untaint the data and routine a safe version of
+it if it validates, while C<valid_> routines simply return a true value if the
+validation succeeds and false otherwise.
 
- # To retrieve all meta data for a field:
- $meta_href = $results->meta('img');
- 
- # Access a particular piece: 
- $width = $results->meta('img')->{width};
- 
-Here's how to set some meta data. This is useful to know if you are
-writing your own complex constraint.
+It is preferable to write "match" routines that untaint data for the extra security
+benefits. Plus, Data::FormValidator will AUTOLOAD a "valid_" version if anyone tries to
+use it, so you only need to write one routine to cover both cases. 
 
-	$self->meta('img', {
-		width  => '50',
-		height => '60',
-	});
+Usually validation routines only need one input, the value being specified. However,
+sometimes more than one value is needed. For that, the following syntax is
+recommended for calling the routines:
 
-This function does not currently multi-valued fields. If it 
-does in the future, the above syntax will still work..
+B<Example>:
 
-=cut
+		image_field  => {  
+			constraint_method  => 'max_image_dimensions',
+			params => [\100,\200],
+		},
 
-sub meta {
-	my $self  = shift;
-	my $field = shift;
-	my $data  = shift;
+Using this syntax, the first parameter that will be passed to the routine is
+the Data::FormValidator object. The remaining parameters will come from the
+C<params> array. Strings will be replaced by the values of fields with the same names,
+and references will be passed directly.
 
-	# initialize if it's the first call
-	$self->{__META} ||= {};
+A couple of of useful methods to use on the Data::FormValidator::Results object  are
+available to you to use inside of your routine.
 
-	if ($data) {
-		(ref $data eq 'HASH') or die 'meta: data passed not a hash ref'; 
-        $self->{__META}{$field} = $data;
-	}
+=over 4
 
+=item get_input_data
 
-	# If we are passed a field, return data for that field
-	if ($field) {
-		return $self->{__META}{$field};
-	}
-	# Otherwise return a list of all fields that have meta data
-	else {
-		return keys %{ $self->{__META} };
-	}
-}
+Returns the raw input data. This may be a CGI object if that's what 
+was used in the validation routine. 
 
-# These are documented in ::Constraints, in the section
-# on writing your own routines. It was more intuitive
-# for the user to look there. 
+B<Example>
+
+ my $data = $self->get_input_data;
+
+=back
+
+=cut 
 
 sub get_input_data {
 	my $self = shift;
 	return $self->{__INPUT_DATA};
 }
 
+=pod
+
+=over 4
+
+=item get_current_constraint_field
+
+Returns the name of the current field being tested in the constraint.
+
+B<Example>:
+
+ my $field = $self->get_current_constraint_field;
+
+This reduces the number of parameters that need to be passed into the routine
+and allows multi-valued constraints to be used with C<constraint_regexp_map>.
+
+=back
+
+For complete examples of multi-valued constraints, see L<Data::FormValidator::Constraints::Upload>
+
+=cut
+
 sub get_current_constraint_field {
 	my $self = shift;
 	return $self->{__CURRENT_CONSTRAINT_FIELD};
 }
+=pod
+
+=over 4
+
+=item get_current_constraint_value
+
+Returns the name of the current value being tested in the constraint.
+
+B<Example>:
+
+ my $value = $self->get_current_constraint_value;
+
+This reduces the number of parameters that need to be passed into the routine
+and allows multi-valued constraints to be used with C<constraint_regexp_map>.
+
+=back
+
+=cut
 
 sub get_current_constraint_value {
 	my $self = shift;
 	return $self->{__CURRENT_CONSTRAINT_VALUE};
 }
 
-sub get_current_constraint_name {
-	my $self = shift;
-	return $self->{__CURRENT_CONSTRAINT_NAME};
-}
-
-
 # INPUT: prefix_string, hash reference
 # Copies the hash and prefixes all keys with prefix_string
 # OUTPUT: hash refence
@@ -704,29 +723,14 @@ sub prefix_hash {
 # (which must start with a slash or "m", not a paren)
 sub _create_sub_from_RE {
 	my $re = shift || return undef;
-	my $untaint_this = shift;
-
-	# This looks like VooDoo to me, but it works. Simplications welcome. -mls 05/26/03 
-	my $return_code = ($untaint_this) ? '; return ($& =~ m/(.*)/s)[0] if defined($`);' : '';
-
 	my $sub;
-	# If it's "qr" style
-	if (substr($re,0,1) eq '(') {
-		$sub = sub { 
-			my $match = $_[0] =~ $re; 
-			if ($untaint_this && defined $-[0]) {
-				return (substr($_[0], $-[0], $+[0] - $-[0]) =~ m/(.*)/s)[0];
-			}
-			else {
-				return $match;
-			}
-		};
-
+	if ($re =~ /^\(/) {
+		$sub = sub { $_[0] =~ $re };
 	}
 	else {
-		$sub = eval 'sub { $_[0] =~ '.$re.$return_code. '}';
+		$sub = eval 'sub { $_[0] =~ '.$re. '}';
+		die "Error compiling regular expression $re: $@" if $@;
 	}
-	die "Error compiling regular expression $re: $@" if $@;
 	return $sub;
 }
 
@@ -759,18 +763,16 @@ sub _arrayify {
 }
 
 # apply filter, modifying %valid by reference
-# We don't bother trying to filter undefined fields.
-# This prevents warnings from Perl. 
 sub _filter_apply {
 	my ($valid,$field,$filter) = @_;
 	die 'wrong number of arguments passed to _filter_apply' unless (scalar @_ == 3);
 	if (ref $valid->{$field} eq 'ARRAY') {
 		for (my $i = 0; $i < @{ $valid->{$field} }; $i++) {
-			$valid->{$field}->[$i] = $filter->( $valid->{$field}->[$i] ) if defined $valid->{$field}->[$i];
+			$valid->{$field}->[$i] = $filter->( $valid->{$field}->[$i] );
 		}
 	}
 	else {
-		$valid->{$field} = $filter->( $valid->{$field} ) if defined $valid->{$field};
+		$valid->{$field} = $filter->( $valid->{$field} );
 	}
 }
 
@@ -794,8 +796,17 @@ sub _constraint_hash_build {
 
 	# Check for regexp constraint
 	if ((ref $c->{constraint} eq 'Regexp')
-			or ( $c->{constraint} =~ m@^\s*(/.+/|m(.).+\2)[cgimosx]*\s*$@ )) {
-		$c->{constraint} = _create_sub_from_RE($c->{constraint},$untaint_this);
+		or ( $c->{constraint} =~ m@^\s*(/.+/|m(.).+\2)[cgimosx]*\s*$@ )) {
+		#If untainting return the match otherwise return result of match
+               my $return_code = ($untaint_this) ? 'return (substr($_[0], $-[0], $+[0] - $-[0]) =~ m/(.*)/s)[0] if defined($-[0]);' : '';
+		
+		   if (ref $c->{constraint} eq 'Regexp') {
+			   $c->{constraint} = sub { $_[0] =~ $c->{constraint}; eval($return_code) };
+		   }
+		   else {
+			   $c->{constraint} = eval 'sub { $_[0] =~ '. $c->{constraint} . ';' . $return_code . '}';
+		   }
+		die "Error compiling regular expression $c->{constraint}: $@" if $@;
 	}
 	# check for code ref
 	elsif (ref $c->{constraint} eq 'CODE') {
@@ -804,25 +815,11 @@ sub _constraint_hash_build {
 	else {
 		# provide a default name for the constraint if we don't have one already
 		$c->{name} ||= $c->{constraint};
-
-		# Save the current constraint name for later
-		$self->{__CURRENT_CONSTRAINT_NAME} = $c->{name};
 		
 		#If untaint is turned on call match_* sub directly. 
 		if ($untaint_this) {
-			my $routine = 'match_'.$c->{constraint};			
-			my $match_sub = *{qualify_to_ref($routine)}{CODE};
-			if ($match_sub) {
-				$c->{constraint} = $match_sub; 
-			}
-			# If the constraint name starts with RE_, try looking for it in the Regexp::Common package
-			elsif ($c->{constraint} =~ m/^RE_/) {
-				$c->{is_method} = 1;
-				$c->{constraint} = eval 'sub { &_create_regexp_common_constraint(@_)}' 
-					|| die "could not create Regexp::Common constraint: $@";
-			} else {
-				die "No untainting constraint found named $c->{constraint}";
-			}
+			$c->{constraint} = *{qualify_to_ref("match_$c->{constraint}")}{CODE} ||
+				die "No untainting constraint found named '$c->constraint'";
 		}
 		else {
 			# try to use match_* first
@@ -835,12 +832,6 @@ sub _constraint_hash_build {
 			elsif (my $valid_sub = *{qualify_to_ref('valid_'.$c->{constraint})}{CODE}) {
 				$c->{constraint} = $valid_sub;
 			}
-			# Load it from Regexp::Common 
-			elsif ($c->{constraint} =~ m/^RE_/) {
-				$c->{is_method} = 1;
-				$c->{constraint} = eval 'sub { return defined &_create_regexp_common_constraint(@_)}' ||
-					die "could not create Regexp::Common constraint: $@";
-			}
 			else {
 				die "No constraint found named '$c->{name}'";
 			}
@@ -887,18 +878,16 @@ sub _constraint_check_match {
 	}
 }
 
-# Figure out whether the data is a hash reference of a param-capable object and return it has a hash
+# Figure out whether the data is a hash reference of a CGI or Apache::Request object and return it has a hash
 sub _get_data {
 	my ($self,$data) = @_;
 	$self->{__INPUT_DATA} = $data;
 	require UNIVERSAL;
-
-    # This checks whether we have an object or not.
-    if (UNIVERSAL::isa($data,'UNIVERSAL') ) {
+	if (UNIVERSAL::isa($data,'CGI') || UNIVERSAL::isa($data,'Apache::Request') ) {
 		my %return;
 		# make sure object supports param()
 		defined($data->UNIVERSAL::can('param')) or
-		croak("Data::FormValidator->validate() or check() called with an object which lacks a param() method!");
+		croak("Data::FormValidator->validate called with CGI or Apache::Request object which lacks a param() method!");
 		foreach my $k ($data->param()){
 			# we expect param to return an array if there are multiple values
 			my @v = $data->param($k);
@@ -913,20 +902,6 @@ sub _get_data {
 }
 
 
-sub _create_regexp_common_constraint  {
-	require Regexp::Common;
-	import  Regexp::Common 'RE_ALL';
-	my $self = shift;
-	my $re_name = $self->get_current_constraint_name;
-	# deference all input
-	my @params = map {$_ = $$_ if ref $_ }  @_;
-
-	no strict "refs";
-	my $re = &$re_name(-keep=>1,@params) || die 'no matching Regexp::Common routine found';
-	return ($self->get_current_constraint_value =~ qr/^$re$/) ? $1 : undef; 
-}
-
-
 1;
 
 __END__
